package com.redhat.training;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Optional;

import org.eclipse.microprofile.config.inject.ConfigProperty;
import org.eclipse.microprofile.rest.client.inject.RestClient;
import org.jboss.logging.Logger;

import jakarta.annotation.PostConstruct;
import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;

@ApplicationScoped
@Path("/jobs")
public class Activator {
    final Logger LOG = Logger.getLogger(Activator.class.getName());

    @ConfigProperty(name = "api.token")
    Optional<String> token;

    @Inject
    @RestClient
    ApiClient k8s;

    @PostConstruct
    public void checkEnv() {
        if (token.isPresent() && !token.get().isEmpty()) {
            LOG.info("Got token from env.");
            return;
        }

        LOG.info("Token not found in env. Trying serviceaccount token.");
        File tf = new File("/var/run/secrets/kubernetes.io/serviceaccount/token");
        if (tf.exists()) {
            try {
                BufferedReader br = new BufferedReader(new FileReader(tf));
                this.token = Optional.of(br.readLine());
                br.close();
                return;
            } catch (IOException ioe) {
                LOG.error("Can not read service account token. This will probably all go to hell.");
            }
        }

        throw new RuntimeException("API Token not set. Can not talk to API.");
    }

    @POST
    public String createJob(JobDescription job) {
        return k8s.createJob(token.get(),
                            job.getNamespace(),
                            job.getName(),
                            new Job(job.getNamespace(),
                                     job.getName(),
                                     job.getCommand()).getApiResource());
    }
}
